# -*- coding: utf-8 -*-
# @Date    : 2021/9/18 19:53
# @Author  : stellahong (stellahong@fuzhi.ai)
# @Desc    :
class Pipeline():
    """
    Input -> Tokenization -> Model Inference -> Post-Processing (task dependent) -> Output
    """
    def __init__(self):
        pass
    
    def train(self):
        pass
    
    def evaluate(self):
        pass
    
    def predict(self):
        pass
